#ifndef _SENT_H
#define _SENT_H

#include "fm33fg0xxa_fl.h"

/*==================================================================================================
*                                       DEFINES AND MACROS
==================================================================================================*/
/* 系统时钟频率 */
#define SYSTEM_APBCLOCK_MHz             (64U)
/* TIM 外设时钟分频 */
#define SENT_TIM_PRES                   (8U)
/* SENT Tick长度 */
#define SENT_TIM_TICK_US                (6U)
/* SENT DATA NIBBLE个数 */
#define SENT_TIM_NIBBLE_DATA_NUMBER     (6U)

/* TIM 溢出计数长度 */
#define TimUpdateWidthCnt               (uint32_t)(65535*SYSTEM_APBCLOCK_MHz/SENT_TIM_PRES)
/* TIM 外设中断参数 */
#define SENT_TIM_IRQn                   TAU_PGL_IRQn 
#define SENT_TIM_IRQHandler             TAU_PGL_IRQHandler 
#define SENT_TIM                        TAU04
/* SYNC Tick计数 56 个 */
#define SENT_TIM_SYNCTICK_CNT           (56U)
/* SYNC 接收诊断长度误差 */
#define SENT_TIM_SYNCPULS_EMAXERR       (1.25)
#define SENT_TIM_SYNCPULS_EMINERR       (0.75)
#define SENT_TIM_SYNC_US                (SENT_TIM_TICK_US * SENT_TIM_SYNCTICK_CNT)
/* SYNC 接收诊断长度转换为 TIM CCR计数 */
#define SENT_TIM_SYNCPULSE_CCR          (uint32_t)(SENT_TIM_SYNC_US*SENT_TIM_APBCLOCK_MHz)
#define SENT_TIM_SYNCPULSE_CCR_MAX      (uint32_t)(SENT_TIM_SYNCPULS_EMAXERR*SENT_TIM_SYNC_US*SYSTEM_APBCLOCK_MHz/SENT_TIM_PRES)
#define SENT_TIM_SYNCPULSE_CCR_MIN      (uint32_t)(SENT_TIM_SYNCPULS_EMINERR*SENT_TIM_SYNC_US*SYSTEM_APBCLOCK_MHz/SENT_TIM_PRES)


/* 消息帧中数据nibble个数 */
#define SENT_TIM_MAX_NIBBLE_DATA        (6U)

#define STATUS_TIM_CAPTURE_FIRST        (0U)
#define STATUS_TIM_CAPTURE_BUSY         (1U)

#define SENT_PAUSE_PULSE_MODE           (1U) /* 暂停段 1U:使能 0U:不使能 */


/*==================================================================================================
                                 STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/

typedef enum
{
    STATUS_SENT_TIM_ERROR_CRC                     = 0x000U,
    STATUS_SENT_TIM_ERROR_CALIBRATION_LENGTH      = 0x001U,
    STATUS_SENT_TIM_ERROR_NIBBLE_VALUE            = 0x002U,
    STATUS_SENT_TIM_ERROR_FALLING_EDGES_NUMBER    = 0X003U,
    STATUS_SENT_TIM_ERROR_SUCCESSIVE_CALIBRATION  = 0X004U,
    STATUS_SENT_TIM_ERROR_TIMEOUT                 = 0X005U,
    STATUS_SENT_TIM_ERROR_DATA_NUMBER             = 0x006U
} SENT_TIM_ErrorStatusType; 

typedef enum
{
    STATUS_SENT_TIM_NO_MESSAGE                = 0x000U,
    STATUS_SENT_TIM_SUCCESS                   = 0x001U,
    STATUS_SENT_TIM_BUSY                      = 0x002U,
    STATUS_SENT_TIM_ERROR                     = 0x003U,
    STATUS_SENT_TIM_BUS_BUSY                  = 0x004U,
    STATUS_SENT_TIM_CRCFAILED                 = 0x005U,
    STATUS_SENT_TIM_CRCPASSED                 = 0x006U,
    STATUS_SENT_TIM_TIMEOUT                   = 0X007U
} SENT_TIM_StatusType;

/**
* @brief     CRC校验类型
*/
#define crc4mode_16_Element_Array          0
#define crc4mode_256_Element_Array         1
#define crc4mode_Element_Array             crc4mode_16_Element_Array

#define SENT_BITXOR(x, y)      (((uint8_t)(x)) ^ ((uint8_t)(y)))

typedef enum
{
    #if(crc4mode_Element_Array == crc4mode_16_Element_Array)
    SENT_RECOMMENDED_IMPLEMENTATION_16_ELEMENT     = 0U,
    SENT_LEGACY_IMPLEMENTATION_16_ELEMENT          = 1U,
    #endif 

    #if(crc4mode_Element_Array == crc4mode_256_Element_Array)
    SENT_RECOMMENDED_IMPLEMENTATION_256_ELEMENT    = 2U,
    SENT_LEGACY_IMPLEMENTATION_256_ELEMENT         = 3U, 
    #endif 
}crc4mode_type;

/**
* @brief    该结构体包括了处理后符合SENT协议的报文数据
*/
typedef struct 
{
    uint32_t          Tick;
    uint32_t          Length;                                 /* 报文长度 Tick */
    uint8_t           DataNibble[SENT_TIM_MAX_NIBBLE_DATA];   /* Data Nibble */
    uint8_t           StatusCommunication;                    /* Status Communication Nibble */
    uint8_t           FastCrc;                                /* CRC */
    bool              MsgJudge;                               /* 报文 true or false */
} SENT_TIM_FastMsgType;


/**
* @brief    在处理数据过程中所处的阶段
*/
typedef enum
{
    STATUS_SENT_TIM_FAST_IDLE               = 0U,
    STATUS_SENT_TIM_FAST_SYNC_CALIB         = 1U,
    STATUS_SENT_TIM_FAST_CHECKED            = 2U, 
    STATUS_SENT_TIM_FAST_RE_SYNC_CALIB      = 3U,
    STATUS_SENT_TIM_FAST_PAUSE_PULSE        = 4U
} SENT_TIM_StatusFastProcessType;

/**
 * @brief   该结构体包括了将定时器数据转换为SENT的半字节值时，数据处理过程中的数据和状态
*/
typedef struct
{
    uint8_t                           NibbleVal[SENT_TIM_MAX_NIBBLE_DATA + 2];  
    uint8_t                           NibbleIndx; 
    uint8_t                           NibbleStream;
    uint32_t                          TimerVal;
    uint32_t                          CalibrationPulseCcr;
    SENT_TIM_StatusFastProcessType    FastState;
    SENT_TIM_StatusType               MsgState;
    SENT_TIM_ErrorStatusType          MsgErrState;
} SENT_TIM_DataProcessType;


void FL_SENT_TIM_RxInit(void);
void FL_SENT_TIM_RxStart(void);
void FL_SENT_TIM_RxStop(void);
void FL_SENT_TIM_RxInthdr(void);
void FL_SENT_TIM_RxHook(void);

#endif 

