/**
  ******************************************************************************
  * @attention
  * Copyright (c) 2022, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)
  * All rights reserved.
  *
  * Processor:                   FM33FG0xxA
  * http:                        http://www.fmdevelopers.com.cn/
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met
  *
  * 1. Redistributions of source code must retain the above copyright notice,
  *    this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *
  * 3. Neither the name of the copyright holder nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
  * 4. To provide the most up-to-date information, the revision of our documents
  *    on the World Wide Web will be the most Current. Your printed copy may be
  *    an earlier revision. To verify you have the latest information avaliable,
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED
    ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
    OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE
    USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

#include "hdiv.h"

/**
  * @brief  初始化对应外设HDIV.
  * @param  void
  * @retval FL_ErrorStatus枚举值
  *         -FL_PASS 初始化成功
  *         -FL_FAIL 初始化失败
  */
FL_ErrorStatus HDIV_Init(void)
{
    return FL_HDIV_Init(HDIV);
}

/**
  * @brief  HDIV 硬件除法器测试
  * @param  void
  * @retval FL_ErrorStatus枚举值
  *         -FL_PASS 测试正确
  *         -FL_FAIL 测试错误
  */
FL_ErrorStatus HDIV_Calculation(void)
{
    int32_t DivisorEnd = 5;
    int16_t Divisor = 2;
    int32_t Quotient;
    int16_t Residue;
    FL_ErrorStatus result = FL_FAIL;

    (void)HDIV_Init();
    
    result = FL_HDIV_Calculation(HDIV, DivisorEnd, Divisor, &Quotient, &Residue);
    if (result == FL_PASS)
    {
        if ((Quotient == (DivisorEnd / Divisor)) && (Residue == (DivisorEnd % Divisor)))
        {
            return FL_PASS; 
        }
    }

    return FL_FAIL;
}



