#include "sent.h"
#include "tau.h"
#include "hdiv.h"

static SENT_TIM_StatusType SENT_TIM_GetFastMsgFromRaw(uint32_t RawMsg, uint16_t Tick, uint8_t *NibbleVal);
static SENT_TIM_StatusType SENT_TIM_GetFastMsgData(void);

static SENT_TIM_DataProcessType SENT_TIM_DataProcess_FastData = {0};
static SENT_TIM_FastMsgType SENT_TIM_ReceiveFastMsg = {0};

/* 定时器输入捕获结果以及捕获状态 */
static uint32_t tempCaptureStatus = STATUS_TIM_CAPTURE_FIRST;
/* SENT 报文长度 */
static uint32_t tempMessageLength = 0;

/** 
  * @brief  SENT 半字节值除法器换算
  * @param  void
  * @retval void
  */
uint32_t sent_nibble_hdiv_Cal(int32_t DivisorEnd, int16_t Divisor)
{
    int32_t tempQuotient;
    int16_t tempResidue;
    uint32_t result;

    FL_HDIV_WriteOperand(HDIV, (uint32_t)DivisorEnd*1000);
    FL_HDIV_WriteDivisor(HDIV, (uint32_t)Divisor);
    while(FL_HDIV_IsActiveFlag_Busy(HDIV) == 0X1){};
    tempQuotient = FL_HDIV_ReadQuotient(HDIV);
    tempResidue = (int16_t)FL_HDIV_ReadResidue(HDIV);

    FL_HDIV_WriteOperand(HDIV, (uint32_t)tempQuotient);
    FL_HDIV_WriteDivisor(HDIV, 10);
    while(FL_HDIV_IsActiveFlag_Busy(HDIV) == 0X1){};
    tempQuotient = FL_HDIV_ReadQuotient(HDIV);
    tempResidue = (int16_t)FL_HDIV_ReadResidue(HDIV);

    if(tempResidue >= 5)
    {
        result = (uint32_t)(tempQuotient + 1);
    }
    else
    {
        result = (uint32_t)(tempQuotient);
    }
    
    return result;
}

/** 
  * @brief  从信号的脉冲宽度中提取快速消息数据
  * @param  void
  * @retval void
  */
static SENT_TIM_StatusType SENT_TIM_GetFastMsgFromRaw(uint32_t RawMsg, uint16_t Tick, uint8_t *NibbleVal)
{
    uint16_t  Val = 0;
    SENT_TIM_StatusType Result; 

    /* This value need to multiply with 100 in oder to prevent float value */
    Val = sent_nibble_hdiv_Cal(RawMsg, SYSTEM_APBCLOCK_MHz*Tick/SENT_TIM_PRES);

    /* Minimum nibble pulse period (transmission value of 0) = 12 clock ticks  */
    /* Maximum nibble pulse period (transmission value of 15) = 12 + 15 = 27 clock ticks  */
    if((Val >= 12U) && (Val <= 27U))
    {
        tempMessageLength += Val;
        *NibbleVal = (uint8_t)(Val-12U);
        Result = STATUS_SENT_TIM_SUCCESS;
    }else
    {
        tempMessageLength = 0;
        Result = STATUS_SENT_TIM_ERROR;
    }
    return Result;
}

/** 
  * @brief  CRC校验
  * @param  void
  * @retval void
  */
static SENT_TIM_StatusType SENT_TIM_FastCrcCheck(crc4mode_type mode, uint8_t* data, uint32_t numNibbles)
{
    SENT_TIM_StatusType Result;

   /* 256 Element Array */
    #if(crc4mode_Element_Array == crc4mode_256_Element_Array)
    static const uint8_t crc_tab_256[256] = {
    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15,
    13, 12, 15, 14, 9, 8, 11, 10, 5, 4, 7, 6, 1, 0, 3, 2,
    7, 6, 5, 4, 3, 2, 1, 0, 15, 14, 13, 12, 11, 10, 9, 8,
    10, 11, 8, 9, 14, 15, 12, 13, 2, 3, 0, 1, 6, 7, 4, 5,
    14, 15, 12, 13, 10, 11, 8, 9, 6, 7, 4, 5, 2, 3, 0, 1,
    3, 2, 1, 0, 7, 6, 5, 4, 11, 10, 9, 8, 15, 14, 13, 12,
    9, 8, 11, 10, 13, 12, 15, 14, 1, 0, 3, 2, 5, 4, 7, 6,
    4, 5, 6, 7, 0, 1, 2, 3, 12, 13, 14, 15, 8, 9, 10, 11,
    1, 0, 3, 2, 5, 4, 7, 6, 9, 8, 11, 10, 13, 12, 15, 14,
    12, 13, 14, 15, 8, 9, 10, 11, 4, 5, 6, 7, 0, 1, 2, 3,
    6, 7, 4, 5, 2, 3, 0, 1, 14, 15, 12, 13, 10, 11, 8, 9,
    11, 10, 9, 8, 15, 14, 13, 12, 3, 2, 1, 0, 7, 6, 5, 4,
    15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0,
    2, 3, 0, 1, 6, 7, 4, 5, 10, 11, 8, 9, 14, 15, 12, 13,
    8, 9, 10, 11, 12, 13, 14, 15, 0, 1, 2, 3, 4, 5, 6, 7,
    5, 4, 7, 6, 1, 0, 3, 2, 13, 12, 15, 14, 9, 8, 11, 10
    };  
    #endif

    /* 16 Element Array */
    #if(crc4mode_Element_Array == crc4mode_16_Element_Array)
    static const uint8_t crc_tab_16[16]  = {0, 13, 7, 10, 14, 3, 9, 4, 1, 12, 6, 11, 15, 2, 8, 5};
    #endif 

    uint32_t i = 0;
    uint8_t tempCS = 0;
    uint8_t CheckSum = 5;

    switch (mode)
    {
        #if(crc4mode_Element_Array == crc4mode_16_Element_Array)
        case SENT_RECOMMENDED_IMPLEMENTATION_16_ELEMENT:
            for (i = 0; i < numNibbles; i++)
            {
                CheckSum = SENT_BITXOR(data[i], crc_tab_16[CheckSum]);
            }
            CheckSum = SENT_BITXOR(0,crc_tab_16[CheckSum]);
            break;
        case SENT_LEGACY_IMPLEMENTATION_16_ELEMENT:
            for (i = 0; i < numNibbles; i++)
            {
                CheckSum = SENT_BITXOR(data[i], crc_tab_16[CheckSum]);
            }
            break;
        #endif 

        #if(crc4mode_Element_Array == crc4mode_256_Element_Array)
        case SENT_RECOMMENDED_IMPLEMENTATION_256_ELEMENT:
            for (i = 0; i < numNibbles; i++)
            {
                tempCS = data[i] + (CheckSum << 4);
                CheckSum = crc_tab_256[tempCS];
            }
            CheckSum = crc_tab_256[(CheckSum << 4)];
            break;
        case SENT_LEGACY_IMPLEMENTATION_256_ELEMENT:
            for (i = 0; i < numNibbles; i++)
            {
                tempCS = data[i] + (CheckSum << 4);
                CheckSum = crc_tab_256[tempCS];
            }
            break;
        #endif 

        default:
            break;
    }

    /* CRC校验已完成 */
    SENT_TIM_DataProcess_FastData.FastState = STATUS_SENT_TIM_FAST_CHECKED;

    if(data[numNibbles] == CheckSum)
    {
        Result = STATUS_SENT_TIM_CRCPASSED;
    }
    else
    {
        Result = STATUS_SENT_TIM_CRCFAILED;
    }
    return Result;
}

static SENT_TIM_StatusType SENT_TIM_GetFastMsgData(void)
{
    static uint8_t temp_DataIndx = 0;
    static SENT_TIM_StatusType RetStatus = STATUS_SENT_TIM_NO_MESSAGE;

    uint32_t tempTimCcr = 0;

    /* 获取 TIM CCR 值 */
    tempTimCcr = FL_TAU_ReadCompare(SENT_TIM);

    switch (SENT_TIM_DataProcess_FastData.FastState)
    {
        /* 查询SYNC，同步SENT报文 */
        case STATUS_SENT_TIM_FAST_IDLE:
            /* 检查可接受范围内的SYNC公差脉冲 +-25% */
            if((SENT_TIM_SYNCPULSE_CCR_MIN <= tempTimCcr) && (tempTimCcr <= SENT_TIM_SYNCPULSE_CCR_MAX))
            {

                if(0 == SENT_TIM_DataProcess_FastData.CalibrationPulseCcr)
                {
                    SENT_TIM_DataProcess_FastData.CalibrationPulseCcr = tempTimCcr;

                    /* 进入报文接收 BUSY 状态 */
                    SENT_TIM_DataProcess_FastData.MsgState = STATUS_SENT_TIM_BUSY;

                }
                else
                {
                    /* 检查校准脉冲上次采样值与当前采样值的差异是否超过1.5625% */
                    if(((SENT_TIM_DataProcess_FastData.CalibrationPulseCcr * 197U) <= (tempTimCcr * 200))
                     &&((tempTimCcr * 200) <= (SENT_TIM_DataProcess_FastData.CalibrationPulseCcr * 203U)))
                    {
                        SENT_TIM_ReceiveFastMsg.MsgJudge = true;

                        /* 上一帧报文接收成功 */
                        SENT_TIM_DataProcess_FastData.MsgState = STATUS_SENT_TIM_SUCCESS;
                    }
                    else
                    {
                        SENT_TIM_ReceiveFastMsg.MsgJudge = false;
                        
                        /* 报文接收 ERROR 状态 */
                        SENT_TIM_DataProcess_FastData.MsgState = STATUS_SENT_TIM_ERROR;
                        /* 报文接收 连续SYNC错误 */
                        SENT_TIM_DataProcess_FastData.MsgErrState = STATUS_SENT_TIM_ERROR_SUCCESSIVE_CALIBRATION;
                    }

                    /* 更新捕获的 SYNC CCR */
                    SENT_TIM_DataProcess_FastData.CalibrationPulseCcr = tempTimCcr;
                    /* 计算当前 SENT Tick时间 */
                    SENT_TIM_ReceiveFastMsg.Tick = (uint32_t)((SENT_TIM_DataProcess_FastData.CalibrationPulseCcr*SENT_TIM_PRES*100)/SYSTEM_APBCLOCK_MHz/56);
                    /* SYNC段同步成功，SENT接收状态切换为对STATUS/DATA/CRC接收 */
                    SENT_TIM_DataProcess_FastData.FastState = STATUS_SENT_TIM_FAST_SYNC_CALIB;
                    /* 清零半字节接收计数 */
                    SENT_TIM_DataProcess_FastData.NibbleIndx = 0;
                }

            }
            else
            {
                /* 报文接收 ERROR 状态 */
                SENT_TIM_DataProcess_FastData.MsgState = STATUS_SENT_TIM_ERROR;
                /* 报文接收 SYNC 长度计算错误 */
                SENT_TIM_DataProcess_FastData.MsgErrState = STATUS_SENT_TIM_ERROR_CALIBRATION_LENGTH;
            }

            /* 数据计数清零 */
            temp_DataIndx = 0;
            break;

        /* 已同步，接收计算半字节数据值 */
        case STATUS_SENT_TIM_FAST_SYNC_CALIB:
            
            SENT_TIM_DataProcess_FastData.TimerVal = tempTimCcr;

            if(STATUS_SENT_TIM_SUCCESS == SENT_TIM_GetFastMsgFromRaw(SENT_TIM_DataProcess_FastData.TimerVal,
                                                                     SENT_TIM_ReceiveFastMsg.Tick,
                                                                     (uint8_t *)(&SENT_TIM_DataProcess_FastData.NibbleStream)))
            {
                /* 接收 Nibble 数据 */
                SENT_TIM_DataProcess_FastData.NibbleVal[temp_DataIndx] = SENT_TIM_DataProcess_FastData.NibbleStream;
                SENT_TIM_DataProcess_FastData.NibbleIndx = temp_DataIndx++;

                /* Check whether this nibble value is CRC: dataIndx = NUMBER_OF_NIBBLES + 2 */
                if(temp_DataIndx == (uint8_t)(SENT_TIM_NIBBLE_DATA_NUMBER + 2U))
                {
                    /* 帧接收完成 */
                    SENT_TIM_DataProcess_FastData.FastState = STATUS_SENT_TIM_FAST_CHECKED;

                    /* SENT 报文长度加 SYNC 56 Tick */
                    tempMessageLength += 56;

                    /* CRC 校验 */
                    if(STATUS_SENT_TIM_CRCPASSED == SENT_TIM_FastCrcCheck(SENT_RECOMMENDED_IMPLEMENTATION_16_ELEMENT,
                                                    SENT_TIM_DataProcess_FastData.NibbleVal + 1U,
                                                    SENT_TIM_NIBBLE_DATA_NUMBER))
                    {
                        /* CRC校验成功后获取快速消息数据 */
                        /* 1. Status and Communication Nibble */
                        SENT_TIM_ReceiveFastMsg.StatusCommunication = SENT_TIM_DataProcess_FastData.NibbleVal[0];
                        /* 2. CRC Nibble */
                        SENT_TIM_ReceiveFastMsg.FastCrc = SENT_TIM_DataProcess_FastData.NibbleVal[SENT_TIM_NIBBLE_DATA_NUMBER + 1U];
                        /* 3. Data Nibble*/
                        for(temp_DataIndx = 0; temp_DataIndx < SENT_TIM_NIBBLE_DATA_NUMBER; temp_DataIndx++)
                        {
                            SENT_TIM_ReceiveFastMsg.DataNibble[temp_DataIndx] = SENT_TIM_DataProcess_FastData.NibbleVal[temp_DataIndx+1U];
                        }

                        #if (SENT_PAUSE_PULSE_MODE == 0U)
                        /* 记录 SENT 报文长度，单位Tick */
                        SENT_TIM_ReceiveFastMsg.Length = tempMessageLength;
                        tempMessageLength = 0;
                        /* 报文接收进入 IDLE ，等待下一帧报文 */
                        SENT_TIM_DataProcess_FastData.FastState = STATUS_SENT_TIM_FAST_IDLE;
                        #endif

                        #if (SENT_PAUSE_PULSE_MODE == 1U)
                        /* 进入暂停段 */
                        SENT_TIM_DataProcess_FastData.FastState = STATUS_SENT_TIM_FAST_PAUSE_PULSE;
                        #endif

                    }
                    else
                    {
                        /* 报文接收 ERROR 状态 */
                        SENT_TIM_DataProcess_FastData.MsgState = STATUS_SENT_TIM_ERROR;
                        /* 报文接收 CRC 计算错误 */
                        SENT_TIM_DataProcess_FastData.MsgErrState = STATUS_SENT_TIM_ERROR_CRC;
                        /* 报文接收进入 IDLE ，等待下一帧报文 */
                        SENT_TIM_DataProcess_FastData.FastState = STATUS_SENT_TIM_FAST_IDLE;

                    }

                    /* 数据计数清零 */
                    temp_DataIndx = 0;

                }
                else if(temp_DataIndx < (uint8_t)(SENT_TIM_NIBBLE_DATA_NUMBER + 2U))
                {
                    SENT_TIM_DataProcess_FastData.MsgState = STATUS_SENT_TIM_BUSY;
                }
                else if(temp_DataIndx > (uint8_t)(SENT_TIM_NIBBLE_DATA_NUMBER + 2U))
                {
                    /* 报文接收 ERROR 状态 */
                    SENT_TIM_DataProcess_FastData.MsgState = STATUS_SENT_TIM_ERROR;
                    /* 报文接收 CRC 计算错误 */
                    SENT_TIM_DataProcess_FastData.MsgErrState = STATUS_SENT_TIM_ERROR_DATA_NUMBER;
                    /* 报文接收进入 IDLE ，等待下一帧报文 */
                    SENT_TIM_DataProcess_FastData.FastState = STATUS_SENT_TIM_FAST_IDLE;
                    /* 数据计数清零 */
                    temp_DataIndx = 0;
                }
            }
            else
            {
                /* 报文接收 ERROR 状态 */
                SENT_TIM_DataProcess_FastData.MsgState = STATUS_SENT_TIM_ERROR;
                /* 报文接收 NIBBLE 值错误 */
                SENT_TIM_DataProcess_FastData.MsgErrState = STATUS_SENT_TIM_ERROR_NIBBLE_VALUE;
                /* 报文接收进入 IDLE ，等待下一帧报文 */
                SENT_TIM_DataProcess_FastData.FastState = STATUS_SENT_TIM_FAST_IDLE;
                /* 数据计数清零 */
                temp_DataIndx = 0;
            }
            break;

        case STATUS_SENT_TIM_FAST_PAUSE_PULSE:
            /* SENT 报文长度加 PAUSE Tick */
            tempMessageLength += sent_nibble_hdiv_Cal(tempTimCcr, SENT_TIM_ReceiveFastMsg.Tick*SYSTEM_APBCLOCK_MHz/SENT_TIM_PRES);

            /* 记录 SENT 报文长度，单位Tick */
            SENT_TIM_ReceiveFastMsg.Length = tempMessageLength;

            /* 报文长度计数清零 单位Tick */
            tempMessageLength = 0;
            /* 数据计数清零 */
            temp_DataIndx = 0;
            /* 报文接收进入 IDLE ，等待下一帧报文 */
            SENT_TIM_DataProcess_FastData.FastState = STATUS_SENT_TIM_FAST_IDLE;

            break;

        default :
            break;
    }

    /* 使能定时器，计数值从0开始重新计数 */
    FL_TAU_Enable(SENT_TIM);

    RetStatus = SENT_TIM_DataProcess_FastData.MsgState;

    return RetStatus;

}

/** 
  * @brief  TAU中断服务函数
  * @param  void
  * @retval void
  */
void SENT_TIM_IRQHandler(void)
{
    static SENT_TIM_StatusType getFastMsgState = STATUS_SENT_TIM_NO_MESSAGE;

    /* 输入捕捉中断(计算周期/频率) */
    if(FL_TAU_IsEnabledIT_IC(SENT_TIM) && FL_TAU_IsActiveFlag_IC(SENT_TIM))
    {
        /* 定时器计数 */
        if(tempCaptureStatus == STATUS_TIM_CAPTURE_BUSY)
        {
            /* 接收快速消息 */
            getFastMsgState = SENT_TIM_GetFastMsgData();
            
            /* SENT 报文接收成功 */
            if(getFastMsgState == STATUS_SENT_TIM_SUCCESS)
            {
                /*********** user code **********/



                /*********** user code **********/
            }

        }
        /* 捕捉到第一个有效沿 ,定时器开始计数 */
        else if(tempCaptureStatus == STATUS_TIM_CAPTURE_FIRST)
        {
            tempCaptureStatus = STATUS_TIM_CAPTURE_BUSY;
        }

        /* 清除捕捉标志 */
        FL_TAU_ClearFlag_IC(SENT_TIM); 

    }

}

/** 
  * @brief  TIMER接收报文，捕获结果和转态初始化
  * @param  void
  * @retval void
  */
static void SENT_TIM_DataProcess_InitDevice(void)
{
    /* Clear Fast Data */
    SENT_TIM_DataProcess_FastData.NibbleIndx       = 0U;
    SENT_TIM_DataProcess_FastData.TimerVal         = 0U;
    SENT_TIM_DataProcess_FastData.NibbleStream     = 0U;
    SENT_TIM_DataProcess_FastData.FastState        = STATUS_SENT_TIM_FAST_IDLE;
    SENT_TIM_DataProcess_FastData.MsgState         = STATUS_SENT_TIM_NO_MESSAGE;
}

void FL_SENT_TIM_RxInit(void)
{
    /* 使能除法器 */
    FL_CMU_EnableGroup2BusClock(FL_CMU_GROUP2_BUSCLK_HDIV);
    /* TIMER接收报文，捕获结果和转态初始化 */
    SENT_TIM_DataProcess_InitDevice();
    /* TIMER 输入捕捉初始化 */
    TAU_IC_PERIOD_Init();
}

void FL_SENT_TIM_RxStart(void)
{
    /* 启动定时器 SENT_TIMER*/
    FL_TAU_Enable(SENT_TIM); 
}

void FL_SENT_TIM_RxStop(void)
{
    /* 关闭定时器 SENT_TIMER*/
    FL_TAU_Disable(SENT_TIM); 
}

