/**
  ******************************************************************************
  * @attention
  * Copyright (c) 2023, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)
  * All rights reserved.
  *
  * Processor:                   FM33FG0xxA
  * http:                        http://www.fmdevelopers.com.cn/
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met
  *
  * 1. Redistributions of source code must retain the above copyright notice,
  *    this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *
  * 3. Neither the name of the copyright holder nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
  * 4. To provide the most up-to-date information, the revision of our documents
  *    on the World Wide Web will be the most Current. Your printed copy may be
  *    an earlier revision. To verify you have the latest information avaliable,
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED
    ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
    OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE
    USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

#include "tau.h"
#include "sent.h"

/** 
  * @brief  TAU输入捕捉初始化(周期模式)
  * @param  void
  * @retval void
  */
void TAU_IC_PERIOD_Init(void)
{
    FL_TAU_InitTypeDef        InitStructer;
    FL_NVIC_ConfigTypeDef     InterruptConfigStruct;
    FL_GPIO_InitTypeDef       GPIO_InitStruct;
  
    /* 初始化引脚功能  数字功能*/
    GPIO_InitStruct.pin           = FL_GPIO_PIN_10;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    GPIO_InitStruct.pull          = FL_GPIO_BOTH_DISABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON2;  /* TI04 */
    FL_GPIO_Init(GPIOG, &GPIO_InitStruct);

    /* 初始化 TAU 配置参数 */
    InitStructer.mode               = FL_TAU_OPERATION_MODE_IC_PERIOD;/* 定时器模式：输入捕获 */
    InitStructer.counterSource      = FL_TAU_COUNTER_SOURCE_INTERNAL; /* 计数源配置：内部APBCLK */
    InitStructer.countEdge          = FL_TAU_COUNTER_EDGE_FALLING;     /* 计数源边缘：上升沿 */
    InitStructer.prescaler          = SENT_TIM_PRES-1;                            /* 计数源分频：80分频 */
    InitStructer.autoReload         = 0xffff-1;                        /* 自动重装载值：10000 */
    InitStructer.outputPolarity     = FL_TAU_OC_POLARITY_NORMAL;      /* 输出极性：正向 */
    InitStructer.compare            = 100-1;                          /* 输出比较值：1000 */
    InitStructer.captureMode        = FL_TAU_IC_MODE_RESET;          /* 输入捕捉模式：自由模式 */
    InitStructer.captureSource      = FL_TAU_IC_SOURCE_GROUP0;        /* 输入捕捉源：GROUP0 */
    InitStructer.captureEdge        = FL_TAU_IC_EDGE_FALLING;          /* 输入捕捉边缘：上升沿 */
    InitStructer.capturePrescaler   = 0;                              /* 输入捕捉源分频：不分频 */
    InitStructer.useOneShotCapture  = FL_DISABLE;                     /* 单次捕捉控制：连续捕捉 */
    InitStructer.useCaptureFilter   = FL_ENABLE;                     /* 输入捕捉滤波：关闭 */
    InitStructer.cascadeMode        = FL_TAU_CASCADE_MODE_DISABLE;    /* 级联模式：关闭 */
    InitStructer.slaveMode          = FL_TAU_SLAVE_MODE_TRIG_CNT;     /* 从机模式：触发递增模式 */
    InitStructer.slaveTriggerSource = FL_TAU_TRGI_GROUP0;             /* 从机触发源：GROUP0 */
  
    FL_TAU_Init(SENT_TIM,&InitStructer);  /* 初始化SENT_TIMER */
    
    FL_TAU_ClearFlag_IC(SENT_TIM);        /* 清除输入捕捉标志 */
    FL_TAU_EnableIT_IC(SENT_TIM);         /* 使能输入捕捉中断 */
    FL_TAU_ClearFlag_Update(SENT_TIM);    /* 清除定时器溢出标志 */
    FL_TAU_EnableIT_Update(SENT_TIM);     /* 使能定时器溢出中断 */

    /* 配置NVIC */
    InterruptConfigStruct.preemptPriority = 0x02;     
    FL_NVIC_Init(&InterruptConfigStruct, SENT_TIM_IRQn);
  
}

